"use strict";
const hrtime_now_1 = require("hrtime-now");
const typescript_ast_util_1 = require("typescript-ast-util");
const typescript_plugin_util_1 = require("typescript-plugin-util");
const codeFixes_1 = require("./codeFixes");
const PLUGIN_NAME = 'typescript-plugin-proactive-code-fixes';
const REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-refactor-action`;
let ts;
let info;
const pluginDefinition = { getApplicableRefactors, getEditsForRefactor };
let target;
let log;
let diagnostics;
function getApplicableRefactors(fileName, positionOrRange) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange) || [];
    const program = info.languageService.getProgram();
    const sourceFile = program.getSourceFile(fileName);
    if (!sourceFile) {
        return refactors;
    }
    diagnostics = typescript_ast_util_1.getDiagnosticsInCurrentLocation(program, sourceFile, typescript_ast_util_1.positionOrRangeToNumber(positionOrRange)) || [];
    if (!log) {
        log = function (msg) {
            info.project.projectService.logger.info(`${PLUGIN_NAME} ${msg}`);
        };
    }
    target = typescript_ast_util_1.findChildContainingRange(sourceFile, typescript_ast_util_1.positionOrRangeToRange(positionOrRange));
    if (!target) {
        info.project.projectService.logger.info(`${PLUGIN_NAME} no getApplicableRefactors because findChildContainedRange  target node is undefined `);
        return refactors;
    }
    const fixes = codeFixes_1.codeFixes.filter(fix => fix.predicate(diagnostics, target, log));
    if (!fixes || !fixes.length) {
        info.project.projectService.logger.info(`${PLUGIN_NAME} no getApplicableRefactors because no codeFixes returned true for node of kind ==${typescript_ast_util_1.getKindName(target.kind)} and diagnostics: ${diagnostics.map(d => d.code)}`);
        return refactors;
    }
    const actions = fixes;
    refactors.push({
        name: `${PLUGIN_NAME}-refactor-info`,
        description: 'Add type',
        actions: fixes.map(fix => ({
            name: REFACTOR_ACTION_NAME + '-' + fix.name,
            description: fix.description(diagnostics, target)
        }))
    });
    info.project.projectService.logger.info(`${PLUGIN_NAME} getApplicableRefactors took ${hrtime_now_1.now() - t0}`);
    return refactors;
}
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName);
    if (!actionName.startsWith(REFACTOR_ACTION_NAME) || !target) {
        return refactors;
    }
    const fixName = actionName.substring(REFACTOR_ACTION_NAME.length + 1, actionName.length);
    const fix = codeFixes_1.codeFixes.find(fix => fix.name === fixName);
    if (!fix) {
        info.project.projectService.logger.info(`${PLUGIN_NAME} no getEditsForRefactor because no fix was found for actionName == ${actionName}`);
        return refactors;
    }
    const simpleProject = typescript_plugin_util_1.createSimpleASTProject(info.project);
    let node = simpleProject.getSourceFile(fileName).getDescendantAtPos(typescript_ast_util_1.positionOrRangeToNumber(positionOrRange));
    if (!node) {
        info.project.projectService.logger.info(`${PLUGIN_NAME} no getEditsForRefactor because getDescentantAt pos returned null for fileName=== ${fileName}, r actionName == ${actionName}`);
        return refactors;
    }
    fix.apply(diagnostics, node, log);
    simpleProject.saveSync();
    simpleProject.emit();
    info.project.projectService.logger.info(`${PLUGIN_NAME} getEditsForRefactor took ${hrtime_now_1.now() - t0}`);
    return refactors;
}
module.exports = typescript_plugin_util_1.getPluginCreate(pluginDefinition, (modules, anInfo) => {
    ts = modules.typescript;
    info = anInfo;
    info.project.projectService.logger.info(`${PLUGIN_NAME} created`);
});
//# sourceMappingURL=index.js.map